;;; my.el --- my own .emacs

;; Copyright (C) 1999 by Stephen Tse
;;
;; Author: Stephen Tse <stephent@sfu.ca>
;; License: GNU Public License
;; Where: http://www.sfu.ca/~stephent/emacs/my.el
;; Updated: Aug 07, 1999
;; Keywords: local

;;; Commentary
;;
;; I publish this .emacs just for sharing. Most stuff here are personal and
;; may not be useful for you. Any feedback or suggestion is welcome.
;;
;; If you want to customize the Windows and Menu keys as alt and hyper on
;; your 104 keywords, visit http://www.jwz.org/xkeycaps/.
;;
;; To install, byte-compile this file and put it in ~/elisp. Then put,
;;   (add-to-list 'load-path "~/elisp")
;;   (require 'my)
;;
;; That's all in my .emacs!
;;

;;; Code:

;; automagically update web copy
;; (write-region (point-min) (point-max) "/stephent@ftp.sfu.ca:~/pub_html/emacs/my.el")

(require 'browse-url)

(eval-when-compile
  (require 'compile)
  (require 'cc-mode)
  (require 'quail))

;;; routine

(defun pbug ()
  "Check parenthesis bugs or similar horrors.

Even with Emacs advanced programming facilities, checking mismatching
parenthesis or missing quote (so called \"pbug\") is no less annoying than
pointer chasing in C.

This function divides the buffer into regions and tries evaluating them one
by one.  It stops at the first region where it fails to evaluate because of
pbug or any other errors. It sets point and mark (and highlights if
`transient-mark-mode' is on) on the failing region and center its first
line.  \"^def\" is used to define regions.  You may also `eval-region'
right after pbug is done to let lisp parse pinpoint the bug.

No more \"End of file during parsing\" horrors!"
  (interactive)
  (let ((point (point))
	(region-regex "^(def..")
	defs beg end)
    (goto-char (point-min))
    (setq defs (loop while (search-forward-regexp region-regex nil t)
		     collect (point-at-bol)))
    ;; so it evals last definition
    (nconc defs (list (point-max)))
    (setq beg (point-min))
    (while defs
      (goto-char beg)
      (setq end (pop defs))
      ;; to be cool, uncomment these to see pbug doing step by step
      ;; (message "checking pbug from %s to %s..." beg end)
      ;; (sit-for 1)
      (when (eq (condition-case nil
		    (eval-region beg (1- end))
		  (error 'pbug-error))
		'pbug-error)
	(push-mark end 'nomsg 'activate)
	(goto-char beg)
	(recenter)
	(error "a pbug found from %s to %s" beg end))
      (setq beg end))
    (goto-char point)
    (message "no pbug found")))
(global-set-key [(super c) p] 'pbug)

(defun hack-let ()
  "Hack let-scoping and bind all its variable forms.

Normally it is a good programming practice to `let' variables in a
function, but in debugging it is convenient to bind _all_ variables so that
`eval-last-sexp' comes handy in places even inside a `let' scope.

Call this inside a `let' scope."
  (interactive)
  (save-excursion
    (let (beg end)
      (end-of-line)
      (search-backward "(let")
      (forward-char 5)
      (setq beg (point))		; beginning of let

      (forward-sexp)
      (setq end (point))		; end of let

    (loop for form in (read (buffer-substring beg end))
	  do (if (listp form)
		 ;; bound variable to evaluated form
		 (set (first form) (eval (second form)))
	       ;; bound variable to nil
	       (set form nil))))))
(global-set-key [(super c) l] 'hack-let)

(defun delete-trailing-whitespace (begin end)
  "Delete trailing whitespace from all lines in region BEGIN and END."
  (save-excursion
    (narrow-to-region begin end)
    (goto-char (point-min))
    (while (re-search-forward "[ \t\r]+$" nil t)
      (replace-match ""))
    (widen)))

(defun just-one-line (begin end)
  "Leave just one blank in region BEGIN and END."
  (interactive)
  (save-excursion
    (narrow-to-region begin end)
    (goto-char (point-min))
    ;; take care if next def start with comments
    (while (re-search-forward "\)[ \t\n]+\n\\(\(\\|;\\)" nil t)
      (replace-match ")\n\n\\1"))))

(defun make-buffer-neat ()
  "Make lisp code neat."
  (interactive)
  (delete-trailing-whitespace (point-min) (point-max))
  (just-one-line (point-min) (point-max))
  (untabify (point-min) (point-max)))

(defun kill-all-buffers ()
  "Kill literally all buffers.
This keeps Emacs from bloating."
  (interactive)
  (if (fboundp 'gnus-group-exit) (gnus-group-exit))
  (loop for x being buffers
    do (kill-buffer x)))

(global-set-key [(shift insert)] 'x-insert-selection)

(defun copy (beg end)
  "Copy current region from BEG to END to X cutbuffer."
  (interactive "r")
  (own-selection (buffer-substring beg end)))
(global-set-key [(control insert)] 'copy)

(defun switch-to-next-buffer ()
  "Switch to next most recent buffer."
  (interactive)
  (switch-to-buffer nil))
(global-set-key [(alt space)] 'switch-to-next-buffer)

(defun exchange-point-and-mark-without-zmacs ()
  "Exchange point and mark without activiating zmacs region.
To activate use `activate-region' \\[activate-region] instead."
  (interactive)
  (exchange-point-and-mark 'dont-activate-region))
(global-set-key [(control x) (control x)]
  'exchange-point-and-mark-without-zmacs)

(defun set-symbol-near-point ()
  "Set symbol near point."
  (interactive)
  (set (intern (symbol-near-point)) (read)))
(global-set-key [(super x) s] 'set-symbol-near-point)

(defun debug-on-error ()
  "Toggle variable `debug-on-error'."
  (interactive)
  (setq debug-on-error (not debug-on-error))
  (message "debug-on-error=%s" debug-on-error))
(global-set-key [(super c) d] 'debug-on-error)

(defmacro shellx (id)
  "Switch to a shell named ID."
  `(lambda ()
     (interactive)
     (if (get-buffer ,id)
	 (switch-to-buffer ,id)
       (shell)
       (rename-buffer ,id)
       (process-kill-without-query (get-buffer-process ,id)))))
(global-set-key [(super s) ?1] (shellx "*jordon*"))
(global-set-key [(super s) ?2] (shellx "*ally*"))
(global-set-key [(super s) ?3] (shellx "*cartor*"))
(global-set-key 'print (shellx "*jordon*"))
(global-set-key 'scroll-lock (shellx "*ally*"))
(global-set-key 'pause (shellx "*cartor*"))

(defun copy-url-at-point ()
  "Copy url at point."
  (interactive)
  (kill-new (url-get-url-at-point)))
(global-set-key [(alt w) u] 'copy-url-at-point)

(defun copy-symbol-near-point ()
  "Copy symbol near point."
  (interactive)
  (kill-new (symbol-near-point)))
(global-set-key [(alt w) s] 'copy-symbol-near-point)

(defun copy-current-line ()
  "Copy current line."
  (interactive)
  (kill-new (buffer-substring (point-at-bol) (point-at-eol))))
(global-set-key [(alt w) l] 'copy-current-line)

(defun buffer-non-readonly ()
  "Make buffer non-readonly."
  (interactive)
  (setq buffer-read-only nil))
(global-set-key [(alt x) q] 'buffer-non-readonly)

(defadvice switch-to-buffer
  (before exising-buffer activate compile)
  "Avoid creating new buffers.
Switch to exisitng buffers only,unless given a prefix argument."
  (interactive
   (list (read-buffer "Switch to buffer: "
		      (other-buffer) (null current-prefix-arg)))))

(defun make-backup-file-name (file)
  "Make backup file name for FILE instead of dumping everywhere."
  (if (file-exists-p (expand-file-name "~/.autosave"))
      (concat (expand-file-name "~/.autosave")
	      (file-name-nondirectory file) "~")
    (concat file "~")))

(defun toggle-menubar ()
  (interactive)
  (set-specifier
   menubar-visible-p
   (not (specifier-instance menubar-visible-p))))
(defun toggle-modeline ()
  (interactive)
  (set-specifier
   has-modeline-p
   (not (specifier-instance has-modeline-p))))
(defun toggle-toolbar ()
  (interactive)
  (set-specifier
   default-toolbar-visible-p
   (not (specifier-instance default-toolbar-visible-p))))
(global-set-key [(alt c) m] 'toggle-modeline)
(global-set-key [(alt c) b] 'toggle-menubar)
(global-set-key [(alt c) t] 'toggle-toolbar)

(defun highlight-my-keyword ()
  "Highlight code documentation keyword."
  (setq font-lock-keywords
	(append font-lock-keywords
		'(("\\bFIXME\\\|\\bHACK\\\|\\bTODO"
		   (0 'font-lock-warning-face t))))))

(defun check-mail ()
  "Check new mail, popup exit message."
  (interactive)
  (set-process-sentinel 
   (start-process "fetchmail" nil "fetchmail")
   (lambda (process change)
     (message (concat "fetchmail " change))
     (sit-for 1))))
(global-set-key [(alt x) ?0] 'check-mail)

(defun exit-emacs-with-gnus ()
  "Save and exit gnus before exiting Emacs."
  (interactive)
  (if (fboundp 'gnus-group-exit) (gnus-group-exit))
  (save-buffers-kill-emacs))
(global-set-key [(control x) (control c)] 'exit-emacs-with-gnus)

(defun real-beep ()
  "Make beep sound ignorning `visibile-bell'."
  (let ((visible-bell nil))
    (beep)))

(defun switch-to-scratch ()
  "Switch to scratch buffer.
Create one in `emacs-lisp-mode' if not exists."
  (interactive)
  (let ((previous (get-buffer "*scratch*")))
    (switch-to-buffer "*scratch*")
    ;; don't change current mode
    (unless previous (emacs-lisp-mode))))
(global-set-key [(alt ?4)] 'switch-to-scratch)

;; finger fix! I keep press control-t when I mean control-y.
(global-set-key [(control t)] 'yank)
(global-set-key [(alt t)] 'transpose-chars)

(defun time-stamp ()
  "Insert a time stamp."
  (interactive "*")
  (insert
   (concat "[" (format-time-string "%b %d, %y" (current-time)) "] ")))
(global-set-key [(alt -)] 'time-stamp)

;; hot file
(defmacro hot-file (filename)
  `(lambda () (interactive) (find-file ,filename)))
(global-set-key [(alt ?1)] (hot-file "~/info/world"))
(global-set-key [(alt ?2)] (hot-file "~/info/memo"))
(global-set-key [(alt ?3)] (hot-file "~/.emacs"))
(global-set-key [(alt ?6)] (hot-file "~/elisp/my.el"))
(global-set-key [(alt ?5)] (hot-file "~/info/data"))

;; hot text
(defun hot-text-sfu ()
  (interactive)
  (insert "/stephent@ftp.sfu.ca:~/"))
(global-set-key [(alt i) s] 'hot-text-sfu)

(defun hot-text-geek ()
  (interactive)
  (insert "linux-geek@sfu.ca"))
(global-set-key [(alt i) l] 'hot-text-geek)

(defun my-compile ()
  (interactive)
  (require 'compile)
  (compile-internal compile-command "No more errors"))
(global-set-key [(control print)] 'my-compile)

(defun next-work-frame ()
  "Skip speedbar and other apps windows."
  (interactive)
  (let ((n (if (and (boundp 'speedbar-frame)
		    (eq (next-frame) speedbar-frame))
	       2 1)))
      (other-frame n)))
(global-set-key [(alt tab)] 'next-work-frame)

(defun hungry-backspace ()
  "Similar to hungry delete in cc-mode."
  (interactive)
  (delete-region 
   (point)
   (progn
     (skip-chars-backward " \t\n")
     (point))))
(global-set-key [(alt backspace)] 'hungry-backspace)

;; FIXME how to automatically apply to every new terminal
;; I am addicted to various modifier keys.
(defun reset-terminal ()
  "Apply everytime to telnet terminal."
  (interactive)
  (global-set-key [(meta ?')] 'undefined)
  (global-set-key [(meta \\)] 'undefined)
  (define-key function-key-map [(meta \])] 'event-apply-hyper-modifier)
  (define-key function-key-map [(meta \\)] 'event-apply-super-modifier)
  (define-key function-key-map [(meta ?')] 'event-apply-alt-modifier))
(global-set-key [(super \\)] 'delete-horizontal-space)
(global-set-key [(control c) t] 'reset-terminal)
(reset-terminal)

(defun touch-buffer ()
  "Touch, to be saved, and force recompile."
  (interactive)
  (set-buffer-modified-p t))
(global-set-key [(control x) t] 'touch-buffer)

(defun match-paren (arg) 
  "Go to the matching parenthesis if on parenthesis otherwise insert %." 
  (interactive "p") 
  (cond ((looking-at "\\s\(") (forward-list 1) (backward-char 1)) 
        ((looking-at "\\s\)") (forward-char 1) (backward-list 1)) 
        (t (self-insert-command (or arg 1)))))
(global-set-key "%" 'match-paren)

(defun zap-to-char-backward (char)
  "`zap-to-char' in backward direction."
  (interactive "cZap back to char: ")
  (zap-to-char -1 char))
(global-set-key [(meta Z)] 'zap-to-char-backward)

;;; c/c++

(push '("\\.h\\'" . c++-mode) auto-mode-alist)

(defun my-cc-mode-hook ()
  (setq c-default-style '((other . "user")))
  (setq c-basic-offset 4)
  (setq fill-column 100)
  ;;(c-toggle-hungry-state 1)
  (turn-on-font-lock)
  (auto-fill-mode))
(add-hook 'c-mode-hook 'my-cc-mode-hook)
(add-hook 'c++-mode-hook 'my-cc-mode-hook)

;;; lisp

(setq checkdoc-spellcheck-documentation-flag t)
(require 'checkdoc)

(defun my-emacs-mode-hook ()
  (turn-on-font-lock)
  (checkdoc-minor-mode 1)
  (auto-fill-mode 1)
  ;; put variables, functions, type defintions all in top menu
  (setq imenu-generic-expression
	'((nil "^\\s-*(def\\(un\\|subst\\|macro\\|advice\
\\|var\\|const\\|custom\
\\|group\\|type\\|struct\\|class\\|ine-condition\\)\
\\s-+\\([-A-Za-z0-9+*|:]+\\)" 2)))   
  (outline-minor-mode 1)
  (setq fill-column 75)  
  (setq comment-column 40))
(add-hook 'emacs-lisp-mode-hook 'my-emacs-mode-hook)

;; useful with checkdoc
(global-set-key [(control alt x)] 'eval-defun)
(global-set-key [(super x) (super e)] 'eval-current-buffer)
(global-set-key [(super x) e] 'eval-region)
(global-set-key [(super tab)] 'lisp-complete-symbol)
(global-set-key [(super x) x] 'edebug-defun)

;;; identity

(setq user-mail-address "stephent@sfu.ca")
(setq user-full-name "Stephen Tse")
;; hack identity
(defun user-uid-hack () 1000)
(fset 'user-uid 'user-uid-hack)
(fset 'user-real-uid 'user-uid-hack)
(defun user-login-name-hack ()
  "Identity hack."
  "stephen")
(fset 'user-login-name 'user-login-name-hack) ; for exim identity
(fset 'user-real-login-name 'user-login-name-hack)

;;; mail

(require 'mime-setup)
(setq mail-user-agent 'message-user-agent)

(setq message-default-headers
      "Bcc: on-fire@home.com\nGcc: nnml:archive\n")
(setq query-user-mail-address nil)
(setq gnus-show-mime t)
(setq gnus-show-mime-method 'gnus-article-preview-mime-message)
(setq gnus-expert-user t)
(setq gnus-secondary-select-methods '((nnml "")))
(setq nnmail-spool-file '("/var/spool/mail/stephen"))
(setq nnmail-split-methods
      '(("vanlug" "^Sender: owner-vanlug")
	("icq" "^Sender: owner-icq-devel@lists.realminfo.com")
	("xemacs" "^Sender: owner-xemacs-beta@xemacs.org")
	("inbox" "")))

(defun add-mail-headers ()
  "Add Reply-To, X-OS, and X-Uptime mail headers."
  (let ((maillist '("linux-geek@sfu.ca"))
	to)
    (save-excursion
      (goto-char (point-min))
      (setq to (mail-header 'to (mail-header-extract)))
      (if (member to maillist)
	  (message-add-header (concat "Reply-To: " to)))))
  (message-add-header
   (concat "X-OS: "
	   (substring (shell-command-to-string "uname -a") 0 -1))
   (concat "X-Uptime: "
	   (substring (shell-command-to-string "uptime") 0 -1))))
(add-hook 'message-send-hook 'add-mail-headers)

(defun my-message-mode-hook ()
  (auto-fill-mode 1))
(add-hook 'message-mode-hook 'my-message-mode-hook)

(defun switch-to-gnus ()
  (interactive)
  (if (buffer-live-p (get-buffer"*Group*"))
      (switch-to-buffer "*Group*")
    (gnus)))
(global-set-key [(alt G)] 'switch-to-gnus)

(setq gnus-asynchronous t)
(setq gnus-use-article-prefetch 5)

;;; font lock

(add-hook
 'font-lock-mode-hook
 (lambda ()
   (highlight-my-keyword)
   (turn-on-lazy-shot)))

(add-hook 'font-lock-mode-hook 'turn-on-lazy-shot)
(setq lazy-shot-stealth-verbose nil)
(setq lazy-shot-verbose nil)

;;; dictionary

;; http://www.in-berlin.de/User/myrkr/dictionary.html
;; http://www.dict.org

;; very handy and useful
(setq dictionary-server "localhost")
(require 'dictionary)
(global-set-key [(alt s)] 'dictionary-search)
(global-set-key [(alt m)] 'dictionary-match-words)

;;; misc

(setq dired-no-confirm '(revert-subdirs))
(setq efs-verbose 0)
(setq initial-major-mode 'emacs-lisp-mode)
(setq package-get-require-signed-base-updates nil)
(setq browse-url-browser-function 'browse-url-netscape)
(setq inhibit-startup-message t)
(setq minibuffer-max-depth nil)
(setq enable-recursive-minibuffers t)
(set-default 'sentence-end-double-space nil)
(setq bookmark-save-flag 1)
(auto-compression-mode 1)
(paren-set-mode 'paren)
(unless (gnuserv-running-p) (gnuserv-start))

(setq w3-delay-image-loads t)
(setq w3-allowed-image-types nil)

;; store efs-set-passwd and eicq-user-password
(load "~/password.el")

;; efs umask is broken? no backup -> umask ok
(setq efs-make-backup-files nil)
;;(setq efs-umask "000")
;;(efs-send-umask "ftp.sfu.ca" "stephent" "000")

(global-set-key [(shift f9)]
  (lambda () (interactive) (point-to-register ?1)))
(global-set-key [(control f9)]
  (lambda () (interactive) (jump-to-register ?1)))
(global-set-key [(shift f10)]
  (lambda () (interactive) (point-to-register ?2)))
(global-set-key [(control f10)]
  (lambda () (interactive) (jump-to-register ?2)))
(global-set-key [(shift f11)]
  (lambda () (interactive) (point-to-register ?3)))
(global-set-key [(control f11)]
  (lambda () (interactive) (jump-to-register ?3)))
(global-set-key [(shift f12)]
  (lambda () (interactive) (point-to-register ?4)))
(global-set-key [(control f12)]
  (lambda () (interactive) (jump-to-register ?4)))

(defun my-text-mode-hook ()
  (auto-fill-mode 1))
(add-hook 'text-mode-hook 'my-text-mode-hook)

(defun telnet-fraser ()
  (interactive)
  (telnet "fraser.sfu.ca"))
(global-set-key [(meta pause)] 'telnet-fraser)

(global-set-key [(alt backspace)]
  (lambda ()
    (interactive)
    (delete-region (point) (point-at-bol))))

(define-key help-mode-map [A] 'apropos)
(global-set-key [(super m)] 'gnus-group-mail)
(global-set-key [(alt x) r] 'rename-buffer)
(global-set-key [(alt o)] 'occur)
(global-set-key [(alt ?\\)] 'speedbar-get-focus)
(global-set-key [(alt x) =] 'what-line)
(global-set-key [(super x) ?4] 'bury-buffer)
(global-set-key [(alt x) b] 'browse-url-at-point)
(global-set-key [(alt b)] 'switch-to-buffer)
(global-set-key [(alt l)] 'add-change-log-entry)
(global-set-key [(control z)] 'undefined)
(global-set-key [(control f4)] 'kill-this-buffer)
(global-set-key [f4] 'bury-buffer)
(global-set-key [(control A)] 'beginning-of-line-text)
(global-set-key [(shift f1)] 'manual-entry)
(define-key help-map [\.] 'find-function)

;;; imenu

(defun imenu-jump-to-symbol-definition (symbol)
  "Jump tp functions, variables, and types defintion.
Use hacked `imenu-generic-expression'."
  (interactive (list (symbol-near-point)))
  (imenu symbol))
(global-set-key [(alt g)] 'imenu-jump-to-symbol-definition)

(global-set-key [insert] 'imenu)

;;; zenirc

; (setq zenirc-server-default "irc.debian.org")
(setq zenirc-nick-default "fire")
(setq zenirc-mode-syntax-table (copy-syntax-table text-mode-syntax-table))
(modify-syntax-entry ?<  "<>" zenirc-mode-syntax-table)
(modify-syntax-entry ?> "><" zenirc-mode-syntax-table)
;; color hack
(add-hook
 'zenirc-mode-hook
 (lambda ()
   (font-lock-mode 1)
   (set-syntax-table zenirc-mode-syntax-table)))
(global-set-key [(alt x) ?/] 'zenirc)

;;; chinese

(setup-chinese-big5-environment)
(setq default-input-method 'chinese-b5-tsangchi)

(unless (eq (console-type) 'tty)
  (set-face-font
   'default
   "-uw-ming-medium-r-normal-fantizi-16-160-75-75-c-160-big5-0"
   'global
   '(mule-fonts) 'prepend))

(defun chinese-decode-buffer ()
  "Decode chinese in buffer.
It ignores `buffer-read-only'."
  (interactive)
  (setq buffer-read-only nil)
  (decode-coding-region (point-min) (point-max) 'big5))
(global-set-key [(alt c) d] 'chinese-decode-buffer)

;; patch
(add-hook
 'quail-mode-hook
 (lambda ()
   (define-key quail-mode-map [? ] 'quail-select-current)))

;; keystroke dictionaries
;; http://www.sfu.ca/~stephent/emacs/clib
(global-set-key [(alt c) ?1] (hot-file "~/code/cinaid/lib/associate"))
(global-set-key [(alt c) ?2] (hot-file "~/code/cinaid/lib/canton"))
(global-set-key [(alt c) ?3] (hot-file "~/code/cinaid/lib/ecdict"))
(global-set-key [(alt c) ?4] (hot-file "~/code/cinaid/lib/simple"))

;;; eicq

;; http://www.sfu.ca/~stephent/eicq
(setq eicq-world-rc-filename "~/info/world")
(setq eicq-log-filename "~/icq/log/log")
(setq eicq-user-alias "fire")
(setq eicq-global-key-prefix [(alt q)])
(global-set-key [(alt q) f] 'world-find)
(setq eicq-coding-system 'big5)
(autoload 'eicq-show-window "eicq" "Show eicq main window." t)
(global-set-key [(alt q) w] 'eicq-show-window)
(add-hook 'eicq-load-hook 'eicq-world-update)

(global-set-key [(super ?1)] (hot-file "~/icq/eicq.el"))
(global-set-key [(super ?2)] (hot-file "~/icq/README"))
(global-set-key [(super ?3)] (hot-file "~/icq/NEWS"))
(global-set-key [(super ?4)] (hot-file "~/icq/udp2tcp.cc"))
(global-set-key [(super ?5)] (hot-file "~/icq/ChangeLog"))
(global-set-key [(super ?6)] (hot-file "~/icq/README.developer"))

;;; appearance

(setq-default mode-line-buffer-identification '(buffer-file-name ("%f") ("%b")))
(set-face-background 'zmacs-region "blue")
(set-face-foreground 'highlight "blue")
(set-face-foreground 'modeline "lightblue")
(set-face-foreground 'modeline-buffer-id "pink")
(set-face-background 'paren-match "blue")

(add-hook
 'dired-load-hook
 (lambda ()
   (set-face-foreground
    'dired-face-directory "lightblue")))

(setq visible-bell t)
;; (load-default-sounds)
(set-specifier default-toolbar-visible-p nil)
(setq use-fancy-toolbar 'next)
(require 'toolbar-fancy)
(set-specifier menubar-visible-p nil)
(set-specifier modeline-shadow-thickness 1)
(set-specifier vertical-divider-always-visible-p nil)
(set-specifier horizontal-scrollbar-visible-p nil)
(set-specifier scrollbar-width 5)
(resize-minibuffer-mode 1)

;;; outline

(global-set-key [(super right)] 'outline-next-visible-heading)
(global-set-key [(super left)] 'outline-previous-visible-heading)
(global-set-key [(super down)] 'outline-forward-same-level)
(global-set-key [(super up)] 'outline-backward-same-level)
(global-set-key [(super prior)] 'outline-up-heading)
(global-set-key [(super insert)] 'show-subtree)
(global-set-key [(super delete)] 'hide-subtree)
(global-set-key [(super end)] 'show-children)
(global-set-key [(alt insert)] 'show-all)
(global-set-key [(alt delete)] 'hide-other)

;;; mp3 player

;; stop: kill `ps axu | grep splay | head -n 1 | cut -c 10-15`
(global-set-key [(control x) kp-0]
  (lambda () (interactive) (shell-command "stop &")))

(defun mp3-feel ()
  "Dired mp3 feel directory."
  (interactive)
  (dired "~/mp3/feel"))
(global-set-key [(control x) kp-9] 'mp3-feel)

(defmacro mp3 (filename)
  "Splay mp3 FILENAME."
  `(lambda () (interactive)
     (shell-command (format "splay %s &" ,filename))))

(global-set-key [(control x) kp-1]
  (mp3 "~/mp3/feel/The\\ One.mp3"))
(global-set-key [(control x) kp-2]
  (mp3 "~/mp3/feel/Air\\ Supply\\ -\\ Making\\ Love\\ \
Out\\ of\\ Nothing\\ At\\ All.mp3"))
(global-set-key [(control x) kp-3]
  (mp3 "~/mp3/feel/Eagles\\ -\\ Hotel\\ California.mp3"))

;;; footer
(provide 'my)


;;; my.el ends here
